
/**
 * @_import
 */
import _isFunction from '@web-native-js/commons/js/isFunction.js';
import _isNumeric from '@web-native-js/commons/js/isNumeric.js';
import _isEmpty from '@web-native-js/commons/js/isEmpty.js';
import _isArray from '@web-native-js/commons/js/isArray.js';
import _following from '@web-native-js/commons/arr/following.js';
import _arrFrom from '@web-native-js/commons/arr/from.js';
import _each from '@web-native-js/commons/obj/each.js';
import ENV from '../ENV.js';

/**
 * Binds a (reactive) list context to the instance.
 * Childnodes will be automatically created/removed per key.
 *
 * @param Element 		el
 * @param array 		items
 * @param function 		addedCallback
 * @param function 		removedCallback
 * @param function 		bind
 *
 * @return Promise
 */
export default function itemize(el, items, addedCallback = null, removedCallback = null, bind = true) {
    // -----------------
    const _set = (key, itemData, isUpdate) => {
        var itemEl;
        if (isUpdate && el.children.length) {
            itemEl = el.querySelector('[' + ENV.params.itemIndexAttribute + '="' + key + '"]');
        } else {
            // --------------
            var partials = el.template.partials[key];
            if (_isEmpty(partials) && _isNumeric(key)) {
                partials = el.template.partials['#'];
            }
            if (_isEmpty(partials)) {
                partials = el.template.partials['default'];
            }
            // --------------
            if (!_isEmpty(partials)) {
                itemEl = partials[0].cloneNode(true);
                var existing = {};
                _arrFrom(el.children).forEach(_el => {
                    existing[_el.getAttribute(ENV.params.itemIndexAttribute)] = _el;
                })
                var following = _following(Object.keys(existing), key + ''/*numeric key needs this*/, true/*length*/)
                    .reduce((closest, _key) => closest || existing[_key], null);
                if (following) {
                    var d = following.before(itemEl);
                } else {
                    var d = el.append(itemEl);
                }
                itemEl.setAttribute(ENV.params.itemIndexAttribute, key);
            }
        }
        if (itemEl) {
            if (_isFunction(addedCallback)) {
                addedCallback(itemEl, itemData, key, isUpdate);
            }
        }
    };
    // -----------------
    const _del = (key) => {
        var itemEl = el.querySelector('[' + ENV.params.itemIndexAttribute + '="' + key + '"]');
        if (itemEl) {
            var rspns;
            if (_isFunction(removedCallback)) {
                rspns = removedCallback(itemEl);
            }
            var remove = () => itemEl.remove();
            if (rspns instanceof Promise) {
                rspns.then(remove).catch(remove);
            } else {
                remove();
            }
        }
    };
    // -----------------
    var isUpdate = el.children.length ? true : false;
    _each(items, (item, key) => _set(item, key, isUpdate));
    if (bind && ENV.trap.observe) {
        if (stub(el).previousBindings) {
            ENV.trap.unobserve(stub(el).previousBindings, null, null, {tags: ['#playui-itemize', itemize]});
        }
        ENV.trap.observe(items, changes => {
            changes.forEach(entry => {
                if (_isArray(items) && entry.name === 'length') {
                    return;
                }
                if (entry.type === 'del') {
                    _del(entry.name);
                } else if (entry.type === 'set' || entry.type === 'def') {
                    _set(entry.name, entry.value, entry.isUpdate);
                }
            });
        }, {tags: ['#playui-itemize', itemize]});
        stub(el).previousBindings = items;
    }
    // -----------------
};

/**
 * Helper
 * 
 * @param Element el 
 * 
 * @return object
 */
export const stub = el => {
    if (!el['.stub']) {
        el['.stub'] = {};
    }
    return el['.stub'];
};
